/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package classesUnion;

import data.Attribute;
import data.Example;
import core.SetOfExamples;
import java.util.List;
import java.util.Objects;

/**
 *
 * @author Mateusz Ledzianowski
 */
public abstract class ClassesUnion extends SetOfExamples {

    /**
     * A decision attribute of class
     */
    private final Attribute decisionAttribute;

    /**
     * A value of decision class which is min (or max) value of examples in the
     * classes union
     */
    private final String decisionValue;

    /**
     * Basic constructor of the classes union
     *
     * @param decisionAttribute
     * @param attributeValue
     */
    public ClassesUnion(Attribute decisionAttribute, String attributeValue) {
        super();
        this.decisionAttribute = decisionAttribute;
        this.decisionValue = attributeValue;
    }

    /**
     * Function calculates a classes union from the given list
     * of examples.
     *
     * @param examples
     */
    protected abstract void calculateClassesUnion(List<Example> examples);

    /**
     * @return hash code of the object
     */
    @Override
    public int hashCode() {
        int hash = super.hashCode();
        hash = 29 * hash + Objects.hashCode(this.getDecisionAttribute());
        hash = 29 * hash + Objects.hashCode(this.getDecisionValue());
        return hash;
    }

    /**
     * @param obj object to compare
     * @return true if objects are equal
     */
    @Override    
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final ClassesUnion other = (ClassesUnion) obj;
        if (!Objects.equals(this.decisionValue, other.decisionValue)) {
            return false;
        }
        return Objects.equals(this.decisionAttribute, other.decisionAttribute);
    }

    /**
     * @return the decisionAttribute
     */
    public Attribute getDecisionAttribute() {
        return decisionAttribute;
    }

    /**
     * @return the decisionValue
     */
    public String getDecisionValue() {
        return decisionValue;
    }
}
