/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package coalition;

import classesUnion.ClassesUnion;
import data.Attribute;
import data.Example;
import java.util.Map;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class InsufficientCoalitions extends SetOfCoalitions {

    /**
     * Basic constructor of the insufficient coalitions
     *
     * @param aopcu
     * @param aoncu
     */
    public InsufficientCoalitions(ClassesUnion aopcu,
            ClassesUnion aoncu) {
        super();
        calculateInsufficientCoalitions(aopcu, aoncu);
    }

    /**
     * Basic constructor of the insufficient coalitions
     *
     * @param approxOfPositiveClassesUnions
     * @param approxOfNegativeClassesUnions
     */
    public InsufficientCoalitions(
            Map<String, ClassesUnion> approxOfPositiveClassesUnions,
            Map<String, ClassesUnion> approxOfNegativeClassesUnions) {
        super();
        Attribute decisionAttribute = approxOfPositiveClassesUnions
                .values().stream().findFirst().orElse(null).getDecisionAttribute();
        if (decisionAttribute == null) {
            decisionAttribute = approxOfNegativeClassesUnions
                    .values().stream().findFirst().orElse(null).getDecisionAttribute();
        }
        for (int i = 0; i < decisionAttribute.getValues().size() - 1; ++i) {
            calculateInsufficientCoalitions(
                    approxOfPositiveClassesUnions.get(
                            decisionAttribute.getValues().get(i + 1)),
                    approxOfNegativeClassesUnions.get(
                            decisionAttribute.getValues().get(i)));
        }
    }

    /**
     * Basic constructor of the insufficient coalitions
     *
     * @param ic
     */
    public InsufficientCoalitions(InsufficientCoalitions ic) {
        super(ic);
    }

    /**
     * Function calculates sufficient coalitions of criteria.
     *
     * @param approxOfPositiveClassesUnions
     * @param approxOfNegativeClassesUnions approximation of negative classes unions
     */
    public final void calculateInsufficientCoalitions(
            ClassesUnion approxOfPositiveClassesUnions,
            ClassesUnion approxOfNegativeClassesUnions) {
        for (Example exU : approxOfPositiveClassesUnions) {
            for (Example exL : approxOfNegativeClassesUnions) {
                Coalition coalition = new Coalition(exL, exU);
                this.add(coalition);
                //System.out.println("(" + exL + ") vs (" + exU + "): " + coalition.toString());
            }
        }
    }
}
