/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package data;

import dbrsaaorc.params.Latex;
import java.util.ArrayList;
import java.util.List;
import utils.LatexHelper;

/**
 * Example class
 *
 * @author Mateusz Ledzianowski
 */
public class Example implements Comparable<Example> {

    /**
     * Size is number of attributes in example
     */
    private final int size;
    /**
     * List of attributes' definitions
     */
    private final List<Attribute> attributes;
    /**
     * List of attributes string values
     */
    private final List<String> attributesValue;
    /**
     * List of min predicted attributes string values
     */
    private final List<String> predictedMinValue;
    /**
     * List of max predicted attributes string values
     */
    private final List<String> predictedMaxValue;
    /**
     * List of logs from prediction fot values
     */
    public final List<String> predictionLogs;

    /**
     * Basic constructor of the example
     *
     * @param attributes - list of example's attributes
     */
    public Example(List<Attribute> attributes) {
        this.size = attributes.size();
        this.attributes = attributes;
        this.attributesValue = new ArrayList<>();
        this.predictedMinValue = new ArrayList<>();
        this.predictedMaxValue = new ArrayList<>();
        this.predictionLogs = new ArrayList<>();
        for (int i = 0; i < size; ++i) {
            this.attributesValue.add(null);
            this.predictedMinValue.add(null);
            this.predictedMaxValue.add(null);
            this.predictionLogs.add(null);
        }
    }

    /**
     * Basic constructor of the example
     *
     * @param attributes - list of example's attributes
     * @param line - string line containing example's values in isf format
     */
    public Example(List<Attribute> attributes, String line) {
        this.size = attributes.size();
        this.attributes = attributes;
        this.attributesValue = new ArrayList<>();
        this.predictedMinValue = new ArrayList<>();
        this.predictedMaxValue = new ArrayList<>();
        this.predictionLogs = new ArrayList<>();
        line = line.replaceAll(",", " ");
        line = line.replaceAll("\t", " ");
        line = line.replaceAll(" +", " ");
        String[] splitLine = line.split(" ");
        if (size != splitLine.length) {
            throw new IllegalArgumentException("Wrong number of attributes in line.");
        }
        for (int i = 0; i < size; ++i) {
            this.attributesValue.add(splitLine[i]);
            this.predictedMinValue.add(null);
            this.predictedMaxValue.add(null);
            this.predictionLogs.add(null);
        }
    }

    /**
     * @return hash code of the object
     */
    @Override
    public int hashCode() {
        int hashCode = 1;
        for (String value : attributesValue) {
            hashCode = 31 * hashCode + value.hashCode();
        }
        return hashCode;
    }

    /**
     * @param other object to compare
     * @return true if objects are equal
     */
    @Override
    public boolean equals(Object other) {
        if (other instanceof Example) {
            Example otherExample = (Example) other;

            if (getSize() != otherExample.getSize()) {
                return false;
            }

            for (int i = 0; i < getSize(); ++i) {
                if (!attributesValue.get(i).equals(otherExample.attributesValue.get(i))) {
                    return false;
                }
            }

            return true;
        }

        return false;
    }

    /**
     * @param latex
     * @return string representation of the object
     */
    public String toString(Latex latex) {
        if (getSize() == 0) {
            return ".";
        }
        StringBuilder sb;
        if (latex != Latex.NONE) {
            sb = new StringBuilder(LatexHelper.equation(
                    LatexHelper.nameToString(attributesValue.get(0)),
                    latex));
        } else {
            sb = new StringBuilder(attributesValue.get(0));
        }
        for (int i = 1; i < getSize(); ++i) {
            if (latex != Latex.NONE) {
                sb.append(" & ");
            } else {
                sb.append(", ");
            }
            if (latex != Latex.NONE && getAttribute(i).isDecision()) {
                sb.append(LatexHelper.equation("\\bm{"
                        + LatexHelper.nameToString(attributesValue.get(i))
                        + "}", latex));
            } else {
                sb.append(LatexHelper.equation(
                        LatexHelper.nameToString(attributesValue.get(i)),
                        latex));
            }
        }
        if (latex != Latex.NONE) {
            sb.append(" \\\\");
        }
        return sb.toString();
    }

    /**
     * @return string representation of the object
     */
    @Override
    public String toString() {
        return toString(Latex.NONE);
    }

    /**
     * @param latex
     * @return string representation of the object
     */
    public String toShortString(Latex latex) {
        if (getSize() == 0) {
            return ".";
        }
        if (latex != Latex.NONE) {
            return LatexHelper.nameToString(attributesValue.get(0));
        } else {
            return attributesValue.get(0);
        }
    }

    /**
     * @return string representation of the object
     */
    public String toShortString() {
        return toShortString(Latex.NONE);
    }

    /**
     * Function compares two objects
     *
     * @param o other object
     * @return the value 0 if this is equal to @param o; a value less than 0 if
     * this is less than @param o; a value greater than 0 if this is greater
     * than @param o
     */
    @Override
    public int compareTo(Example o) {
        int cmp = getDecisionAttribute().compareValues(getDecisionValue(),
                o.getDecisionValue());
        if (cmp != 0) {
            return cmp;
        }
        try {
            return Double.valueOf(getAttributeValue(0))
                    .compareTo(Double.valueOf(o.getAttributeValue(0)));
        } catch (NumberFormatException ex) {
            return getAttributeValue(0).compareTo(o.getAttributeValue(0));
        }
    }

    /**
     * @param o example which could by dominated by this example
     * @return true if this example dominate example o
     */
    public boolean dominate(Example o) {
        int n = getSize();
        if (n != o.getSize()) {
            throw new IllegalArgumentException("Both examples should have "
                    + "the same sizes!");
        }
        for (int i = 0; i < n; ++i) {
            Attribute attr = getAttribute(i);
            if (attr.getType() != AttributeType.OMIT && !attr.isDecision()) {
                Integer cmp_attr = attr.compareValues(getAttributeValue(i), o.getAttributeValue(i));
                if (cmp_attr != null && cmp_attr < 0) {
                    return false;
                }
            }
        }
        return true;
    }

    public void clearPrediction() {
        predictedMinValue.clear();
        predictedMaxValue.clear();
        predictionLogs.clear();
        for (int i = 0; i < size; ++i) {
            predictedMinValue.add(null);
            predictedMaxValue.add(null);
            predictionLogs.add(null);
        }
    }

    /**
     * @return first decision string value
     */
    public String getDecisionValue() {
        return getAttributeValue(getDecisionAttribute());
    }

    /**
     * @return first decision attribute position
     */
    public int getDecisionAttributePosition() {
        for (int i = 0; i < getSize(); ++i) {
            if (attributes.get(i).isDecision()) {
                return i;
            }
        }
        return -1;
    }

    /**
     * @return first decision attribute
     */
    public Attribute getDecisionAttribute() {
        int i = getDecisionAttributePosition();
        if (i != -1) {
            return attributes.get(i);
        }
        return null;
    }

    /**
     * @return position of first decision's value (bigger position means better
     * decision class); -1 if there is not a decision attribute.
     */
    public int getDecisionValuePosition() {
        return getDecisionAttribute().getValuePosition(getDecisionValue());
    }

    /**
     * @param attribute which index will be returned
     * @return index of the attribute on list of attributes
     */
    public Integer getIndexOf(Attribute attribute) {
        for (int i = 0; i < getSize(); ++i) {
            if (attributes.get(i).equals(attribute)) {
                return i;
            }
        }
        return null;
    }

    /**
     * @param index of the attribute on list of attributes
     * @return the attribute
     */
    public Attribute getAttribute(int index) {
        return attributes.get(index);
    }

    /**
     * @param index of the attribute on list of attributes
     * @return the attribute type
     */
    public final AttributeType getAttributeType(int index) {
        return attributes.get(index).getType();
    }

    /**
     * @param index of the attribute on list of attributes
     * @return the attribute string value
     */
    public String getAttributeValue(int index) {
        return attributesValue.get(index);
    }

    /**
     * @param attribute which value will be returned
     * @return the attribute string value
     */
    public String getAttributeValue(Attribute attribute) {
        return getAttributeValue(getIndexOf(attribute));
    }

    /**
     * @param index of the attribute on list of attributes
     * @param value to set
     */
    public void setAttribute(int index, String value) {
        attributesValue.set(index, value);
    }

    /**
     * @param attribute which value will be set
     * @param value to set
     */
    public void setAttribute(Attribute attribute, String value) {
        int index = getIndexOf(attribute);
        setAttribute(index, value);
    }

    /**
     * @param index of the attribute on list of attributes
     * @return the attribute predicted string value
     */
    public String getPredictedMinValue(int index) {
        return predictedMinValue.get(index);
    }

    /**
     * @param attribute which predicted value will be returned
     * @return the attribute predicted string value
     */
    public String getPredictedMinValue(Attribute attribute) {
        return getPredictedMinValue(getIndexOf(attribute));
    }

    /**
     * @param index of the attribute on list of attributes
     * @return the attribute predicted string value
     */
    public String getPredictedMaxValue(int index) {
        return predictedMaxValue.get(index);
    }

    /**
     * @param attribute which predicted value will be returned
     * @return the attribute predicted string value
     */
    public String getPredictedMaxValue(Attribute attribute) {
        return getPredictedMaxValue(getIndexOf(attribute));
    }

    /**
     * @param index of the attribute on list of attributes
     * @param minValue to set
     * @param maxValue to set
     * @param predictionLog
     */
    public void setPredictedValue(int index, String minValue, String maxValue, String predictionLog) {
        predictedMinValue.set(index, minValue);
        predictedMaxValue.set(index, maxValue);
        predictionLogs.set(index, predictionLog);
    }

    /**
     * @param attribute which predicted value will be set
     * @param minValue to set
     * @param maxValue to set
     * @param predictionLog
     */
    public void setPredictedValue(Attribute attribute, String minValue, String maxValue, String predictionLog) {
        int index = getIndexOf(attribute);
        setPredictedValue(index, minValue, maxValue, predictionLog);
    }

    /**
     * @return the number of attributes in example
     */
    public int getSize() {
        return size;
    }

    /**
     * @return the number of attributes without decisions, omits or unordered
     * lists
     */
    public int getSufficientSize() {
        return Attribute.getSignificantAttributes(attributes).size();
    }

    public String getPredictionLog(int index) {
        return predictionLogs.get(index);
    }

    public String getPredictedDecisionValueString() {
        int i = getDecisionAttributePosition();
        Attribute decision = getDecisionAttribute();

        if (predictedMinValue.get(i) == null && predictedMaxValue.get(i) == null) {
            if (decision.getValues().size() >= 2) {
                return String.format("[%s,%s]", decision.getValues().get(0), decision.getValues().get(decision.getValues().size() - 1));
            } else {
                return decision.getValues().get(0);
            }
        } else if (predictedMinValue.get(i) == null) {
            return String.format("%s", predictedMaxValue.get(i));
        } else if (predictedMaxValue.get(i) == null) {
            return String.format("%s", predictedMinValue.get(i));
        } else if (predictedMinValue.get(i).equals(predictedMaxValue.get(i))) {
            return String.format("%s", predictedMaxValue.get(i));
        } else {
            return String.format("[%s, %s]", predictedMinValue.get(i), predictedMaxValue.get(i));
        }
    }

    public String getPredictedDecisionRulesString() {
        int i = getDecisionAttributePosition();
        if (predictionLogs.get(i) == null) {
            return "";
        } else {
            return predictionLogs.get(i);
        }
    }

    public float getPredictionError(Attribute attr) {
        int trueIdx = attr.getValuePosition(getAttributeValue(attr)); // 3, 2, 2
        int predMinIdx = attr.getValuePosition(getPredictedMinValue(attr)); // 1, 2, 4
        int predMaxIdx = attr.getValuePosition(getPredictedMaxValue(attr)); // 5, 2, 5
        if (getPredictedMinValue(attr) == null && getPredictedMaxValue(attr) == null) {
            predMinIdx = 0;
            predMaxIdx = attr.getValues().size() - 1;
        } else if (getPredictedMinValue(attr) == null) {
            predMinIdx = predMaxIdx;
        } else if (getPredictedMaxValue(attr) == null) {
            predMaxIdx = predMinIdx;
        }

        int predRange = predMaxIdx - predMinIdx + 1; // 5, 1, 2

        int predMinErrorRange = trueIdx - predMinIdx; // 2, 0, -2
        int predMaxErrorRange = predMaxIdx - trueIdx; // 2, 0, 3

        float predMinError = (float) (predMinErrorRange * (predMinErrorRange + 1)) / 2; // 3, 0, 1
        if (trueIdx < predMinIdx) {
            predMinError = -predMinError;
        }
        float predMaxError = (float) (predMaxErrorRange * (predMaxErrorRange + 1)) / 2; // 3, 0, 6
        if (predMaxIdx < trueIdx) {
            predMaxError = -predMaxError;
        }

        return (predMinError + predMaxError) / predRange; // 1.2, 0, 2.5
    }

    public float getPredictionError() {
        return getPredictionError(getDecisionAttribute());
    }

    public boolean isPredictionCorrect() {
        return getDecisionAttribute()
                .compareValues(getDecisionValue(),
                        getPredictedMinValue(getDecisionAttribute())) >= 0
                && getDecisionAttribute()
                        .compareValues(getDecisionValue(),
                                getPredictedMaxValue(getDecisionAttribute())) <= 0;
    }
}
