/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dbrsaaorc;

import data.Attribute;
import data.Example;
import dbrsaaorc.params.Latex;
import domlem.ListOfRules;
import java.util.List;
import java.util.Map;
import utils.Lang;
import utils.LatexHelper;

/**
 *
 * @author Mateusz Ledzianowski
 */
public abstract class Classifier {

    protected Map<String, ListOfRules> positiveRules;
    protected Map<String, ListOfRules> negativeRules;
    protected List<Example> examples;
    protected String log;

    /**
     *
     * @param positiveRules
     * @param negativeRules
     * @param examples
     */
    public Classifier(Map<String, ListOfRules> positiveRules, Map<String, ListOfRules> negativeRules, List<Example> examples) {
        this.positiveRules = positiveRules;
        this.negativeRules = negativeRules;
        this.examples = examples;
    }

    public abstract void classify(Latex latex);

    public String getRulesString() {
        StringBuilder sb = new StringBuilder();
        for (ListOfRules rules : getNegativeRules().values()) {
            sb.append(rules.toString());
            sb.append("\n");
        }
        for (ListOfRules rules : getPositiveRules().values()) {
            sb.append(rules.toString());
            sb.append("\n");
        }
        return sb.toString();
    }

    public float getAverageError() {
        float error = 0;
        for (Example ex : examples) {
            error += ex.getPredictionError();
        }
        return error / examples.size();
    }

    public float getAccuracyRatio() {
        float correct = 0;
        for (Example ex : examples) {
            if (ex.isPredictionCorrect()) {
                ++correct;
            }
        }
        return correct / examples.size();
    }

    public Map<String, ListOfRules> getPositiveRules() {
        return positiveRules;
    }

    public void setPositiveRules(Map<String, ListOfRules> positiveRules) {
        this.positiveRules = positiveRules;
    }

    public Map<String, ListOfRules> getNegativeRules() {
        return negativeRules;
    }

    public void setNegativeRules(Map<String, ListOfRules> negativeRules) {
        this.negativeRules = negativeRules;
    }

    public List<Example> getExamples() {
        return examples;
    }

    public void setExamples(List<Example> examples) {
        this.examples = examples;
    }

    public String getLog() {
        return log;
    }

    public String toString(Latex latex, String labelSuff) {
        StringBuilder sb = new StringBuilder();
        if (latex != Latex.NONE && !examples.isEmpty()) {
            int attrSize = examples.get(0).getSize();
            sb.append("\\begin{table}[H]\n"
                    + "\\centering\n"
                    + "\\begin{tabularx}{\\textwidth}{@{}");
            if (attrSize <= 4) {
                sb.append(LatexHelper.repeat("X", attrSize));
                sb.append(LatexHelper.repeat("l", 2));
            } else {
                sb.append(LatexHelper.repeat("l", attrSize + 1));
                sb.append("X");
            }
            sb.append("@{}}\n"
                    + "\\toprule\n");
            for (int i = 0; i < attrSize; ++i) {
                Attribute attr = examples.get(0).getAttribute(i);
                if (attr.isDecision()) {
                    sb.append(LatexHelper.equation("\\bm{"
                            + attr.getName(latex) + "}", latex));
                } else {
                    sb.append(LatexHelper.equation(attr.getName(latex), latex));
                }
                sb.append(" & ");
            }
            sb.append(Lang.prediction(latex));
            sb.append(" & ");
            sb.append(Lang.predRules(latex));
            sb.append(" \\\\ \\midrule\n");
            for (Example example : getExamples()) {
                sb.append(example.toString(latex));
                sb.setLength(sb.length() - 3);
                sb.append(" & ");
                sb.append(LatexHelper.equation(
                        example.getPredictedDecisionValueString(), latex));
                sb.append(" & ");
                sb.append(example.getPredictedDecisionRulesString());
                sb.append(" \\\\\n");
            }
            sb.setLength(sb.length() - 1);
            sb.append(" \\bottomrule\n"
                    + "\\end{tabularx}\n"
                    + "\\caption{" + Lang.classificationCaption(latex) + ".}\n"
                    + "\\label{classification-" + labelSuff + "}\n"
                    + "\\end{table}\n");
        } else {
            sb.append("Classification:\n");
            for (Example ex : examples) {
                Attribute decision = ex.getDecisionAttribute();
                sb.append(String.format("%s: ",
                        ex.toString(latex)));
                sb.append(LatexHelper.equation(
                        String.format("%s=%s",
                                decision.getName(latex),
                                ex.getPredictedDecisionValueString()), latex));
                sb.append(String.format(" %s\n",
                        ex.getPredictedDecisionRulesString()));
            }
        }
        return sb.toString();
    }

    @Override
    public String toString() {
        return toString(Latex.NONE, "");
    }
}
