/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dbrsaaorc;

import classesUnion.ClassesUnion;
import core.SetOfExamples;
import data.Attribute;
import data.Example;
import data.ISF;
import dbrsaaorc.params.Latex;
import domlem.ListOfRules;
import domlem.Rule;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class ClassifierNew extends Classifier {

    public ISF dbModel;
    public Map<String, ClassesUnion> positiveClassesUnions;
    public Map<String, ClassesUnion> negativeClassesUnions;

    public ClassifierNew(Map<String, ListOfRules> positiveRules, Map<String, ListOfRules> negativeRules, ISF dbModel, Map<String, ClassesUnion> upperApproxOfPositiveClassesUnions, Map<String, ClassesUnion> upperApproxOfNegativeClassesUnions, List<Example> examples) {
        super(positiveRules, negativeRules, examples);
        this.dbModel = dbModel;
        this.positiveClassesUnions = upperApproxOfPositiveClassesUnions;
        this.negativeClassesUnions = upperApproxOfNegativeClassesUnions;
    }

    @Override
    public void classify(Latex latex) {
        StringBuilder sb = new StringBuilder();
        Attribute decisionAttr = dbModel.getDecisionAttribute();
        SetOfExamples modelExamples = new SetOfExamples(dbModel.getExamples());
        Map<String, SetOfExamples> clExamples = new HashMap<>();
        for (String decVal : dbModel.getDecisionAttribute().getValues()) {
            clExamples.put(decVal, new SetOfExamples());
        }
        for (Example ex : dbModel.getExamples()) {
            clExamples.get(ex.getDecisionValue()).add(ex);
        }
        for (Example ex : examples) {
            float bestScore = -Float.MAX_VALUE;
            String bestClass;
            String predValue = null;
            List<Long> predRulesIdFor = new ArrayList<>();
            List<Long> predRulesIdAgainst = new ArrayList<>();

            ex.clearPrediction();

            for (String decVal : dbModel.getDecisionAttribute().getValues()) {
                SetOfExamples condAndClExamplesFor = new SetOfExamples();
                SetOfExamples condAndClExamplesAgainst = new SetOfExamples();
                SetOfExamples condExamplesFor = new SetOfExamples();
                SetOfExamples condExamplesAgainst = new SetOfExamples();
                SetOfExamples unionsExamplesAgainst = new SetOfExamples();
                List<Long> rulesIdFor = new ArrayList<>();
                List<Long> rulesIdAgainst = new ArrayList<>();

                for (ListOfRules rules : negativeRules.values()) {
                    for (Rule rule : rules) {
                        if (rule.isFulfilled(ex)) {
                            if (decisionAttr.compareValues(decVal, rule.getDecision().getAttributeValue()) <= 0) {
                                for (Example exModel : rule.getRuleScore(modelExamples).getExamplesInCondition()) {
                                    if (decisionAttr.compareValues(decVal, exModel.getDecisionValue()) == 0) {
                                        condAndClExamplesFor.add(exModel);
                                    }
                                    condExamplesFor.add(exModel);
                                }
                                rulesIdFor.add(rule.getId());
                            } else {
                                for (Example exModel : rule.getRuleScore(modelExamples).getExamplesInCondition()) {
                                    if (decisionAttr.compareValues(decVal, exModel.getDecisionValue()) > 0) {
                                        condAndClExamplesAgainst.add(exModel);
                                    }
                                    condExamplesAgainst.add(exModel);
                                }
                                for (Example exMode : negativeClassesUnions.get(rule.getDecision().getAttributeValue())) {
                                    unionsExamplesAgainst.add(exMode);
                                }
                                rulesIdAgainst.add(rule.getId());
                            }
                        }
                    }
                }
                for (ListOfRules rules : positiveRules.values()) {
                    for (Rule rule : rules) {
                        if (rule.isFulfilled(ex)) {
                            if (decisionAttr.compareValues(decVal, rule.getDecision().getAttributeValue()) >= 0) {
                                for (Example exModel : rule.getRuleScore(modelExamples).getExamplesInCondition()) {
                                    if (decisionAttr.compareValues(decVal, exModel.getDecisionValue()) == 0) {
                                        condAndClExamplesFor.add(exModel);
                                    }
                                    condExamplesFor.add(exModel);
                                }
                                rulesIdFor.add(rule.getId());
                            } else {
                                for (Example exModel : rule.getRuleScore(modelExamples).getExamplesInCondition()) {
                                    if (decisionAttr.compareValues(decVal, exModel.getDecisionValue()) < 0) {
                                        condAndClExamplesAgainst.add(exModel);
                                    }
                                    condExamplesAgainst.add(exModel);
                                }
                                for (Example exMode : positiveClassesUnions.get(rule.getDecision().getAttributeValue())) {
                                    unionsExamplesAgainst.add(exMode);
                                }
                                rulesIdAgainst.add(rule.getId());
                            }
                        }
                    }
                }
                if (condAndClExamplesFor.size() > 0) {
                    float positiveScore = (float) condAndClExamplesFor.size() * condAndClExamplesFor.size() / condExamplesFor.size() / clExamples.get(decVal).size();
                    float negativeScore = 0;
                    if (condAndClExamplesAgainst.size() > 0) {
                        negativeScore = (float) condAndClExamplesAgainst.size() * condAndClExamplesAgainst.size() / condExamplesAgainst.size() / unionsExamplesAgainst.size();
                    }
                    float score = positiveScore - negativeScore;

                    sb.append(String.format("%s = %.2f", scoreString(true, false, decVal, ex, condAndClExamplesFor, condExamplesFor, clExamples.get(decVal)), positiveScore));
                    sb.append("\n");
                    sb.append(String.format("%s = %.2f", scoreString(false, true, decVal, ex, condAndClExamplesAgainst, condExamplesAgainst, unionsExamplesAgainst), negativeScore));
                    sb.append("\n");
                    sb.append(String.format("%s = %.2f", scoreString(false, false, decVal, ex), score));
                    sb.append("\n");

                    if (score > bestScore) {
                        bestClass = decVal;
                        bestScore = score;
                        predValue = bestClass;
                        predRulesIdFor = rulesIdFor;
                        predRulesIdAgainst = rulesIdAgainst;
                    }
                }
            }
            savePrediction(ex, decisionAttr, predValue, predRulesIdFor, predRulesIdAgainst, latex);
            sb.append("\n");
        }
        log = String.format("New classification algorithm:\n  average class error = %.2f\n  accuracy ratio = %.2f\n\n", getAverageError(), getAccuracyRatio());
        if (sb.length() > 1) {
            log += sb.substring(0, sb.length() - 2);
        }
    }

    private String scoreString(boolean positive, boolean negative, String decVal, Example ex) {
        StringBuilder sb = new StringBuilder();
        sb.append("Score");
        if (positive) {
            sb.append("+");
        } else if (negative) {
            sb.append("-");
        }
        sb.append(String.format("(%s,%s)", decVal, ex.toShortString()));
        return sb.toString();
    }

    private String scoreString(boolean positive, boolean negative, String decVal, Example ex, SetOfExamples condAndClExamples, SetOfExamples condExamples, SetOfExamples clExamples) {
        StringBuilder sb = new StringBuilder(scoreString(positive, negative, decVal, ex));
        sb.append(" = ");
        sb.append(String.format("|%s|^2 / ", condAndClExamples));
        sb.append(String.format("|%s||%s|", condExamples, clExamples));
        return sb.toString();
    }

    private void savePrediction(Example ex, Attribute attr, String predValue, List<Long> predRulesIdFor, List<Long> predRulesIdAgainst, Latex latex) {
        if (predValue == null) {
            ex.setPredictedValue(
                    attr,
                    ex.getDecisionAttribute().getValues().get(0),
                    ex.getDecisionAttribute().getValues().get(
                            ex.getDecisionAttribute().getValues().size() - 1),
                    getPredictionLog(predRulesIdFor, predRulesIdAgainst, latex));
        } else {
            ex.setPredictedValue(
                    attr,
                    predValue,
                    predValue,
                    getPredictionLog(predRulesIdFor, predRulesIdAgainst, latex));
        }
    }

    private String getPredictionLog(List<Long> predRulesIdFor, List<Long> predRulesIdAgainst, Latex latex) {
        if (predRulesIdFor.isEmpty() && predRulesIdAgainst.isEmpty()) {
            return "";
        } else {
            StringBuilder sb = new StringBuilder("(");
            if (!predRulesIdFor.isEmpty()) {
                sb.append("+: ");
                sb.append(listToString(predRulesIdFor, latex));
                if (!predRulesIdAgainst.isEmpty()) {
                    sb.append(") (-: ");
                    sb.append(listToString(predRulesIdAgainst, latex));
                }
            } else {
                sb.append("(-: ");
                sb.append(listToString(predRulesIdAgainst, latex));
            }
            return sb.toString() + ")";
        }
    }

    private String listToString(List<Long> predRulesId, Latex latex) {
        StringBuilder sb = new StringBuilder();
        for (Long id : predRulesId) {
            sb.append(id);
            if (latex != Latex.NONE) {
                sb.append(",\\allowbreak ");
            } else {
                sb.append(",");
            }
        }
        if (latex != Latex.NONE) {
            return sb.substring(0, sb.length() - 13);
        } else {
            return sb.substring(0, sb.length() - 1);
        }
    }
}
