/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dbrsaaorc;

import data.Attribute;
import data.Example;
import dbrsaaorc.params.Latex;
import domlem.ListOfRules;
import domlem.Rule;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class ClassifierStandard extends Classifier {

    public ClassifierStandard(Map<String, ListOfRules> positiveRules, Map<String, ListOfRules> negativeRules, List<Example> examples) {
        super(positiveRules, negativeRules, examples);
    }

    @Override
    public void classify(Latex latex) {
        StringBuilder sb = new StringBuilder();
        for (Example ex : examples) {
            Attribute decisionAttr = ex.getDecisionAttribute();
            String predMinValue = null;
            String predMaxValue = null;
            Long predMinRuleId = null;
            Long predMaxRuleId = null;
            
            ex.clearPrediction();
            
            for (ListOfRules rules : negativeRules.values()) {
                for (Rule rule : rules) {
                    if (rule.isFulfilled(ex)) {
                        sb.append(logString(ex, rule));
                        sb.append("\n");
                        String decisionValue = rule.getDecision().getAttributeValue();
                        if (predMaxValue == null || decisionAttr.compareValues(decisionValue, predMaxValue) < 0) {
                            predMaxValue = decisionValue;
                            predMaxRuleId = rule.getId();
                        }
                    }
                }
            }
            for (ListOfRules rules : positiveRules.values()) {
                for (Rule rule : rules) {
                    if (rule.isFulfilled(ex)) {
                        sb.append(logString(ex, rule));
                        sb.append("\n");
                        String decisionValue = rule.getDecision().getAttributeValue();
                        if (predMinValue == null || decisionAttr.compareValues(decisionValue, predMinValue) > 0) {
                            predMinValue = decisionValue;
                            predMinRuleId = rule.getId();
                        }
                    }
                }
            }
            savePrediction(ex, decisionAttr, predMinValue, predMaxValue, predMinRuleId, predMaxRuleId);
            sb.append("\n");
        }
        log = String.format("Standard classification algorithm:\n  average class error = %.2f\n  accuracy ratio = %.2f\n\n", getAverageError(), getAccuracyRatio());
        if (sb.length() > 0) {
            log += sb.substring(0, sb.length() - 1);
        }
    }

    private String logString(Example ex, Rule rule) {
        StringBuilder sb = new StringBuilder();
        sb.append(ex.toShortString());
        sb.append(": ");
        sb.append(rule.getDecision().toShortString());
        sb.append(String.format(" (%d)", rule.getId()));
        return sb.toString();
    }

    private void savePrediction(Example ex, Attribute attr, String predMinValue, String predMaxValue, Long predMinRuleId, Long predMaxRuleId) {
        String savePredMinValue;
        String savePredMaxValue;
        if (predMinValue == null)
            savePredMinValue = ex.getDecisionAttribute().getValues().get(0);
        else
            savePredMinValue = predMinValue;
        if (predMaxValue == null)
            savePredMaxValue = ex.getDecisionAttribute().getValues().get(ex.getDecisionAttribute().getValues().size() - 1);
        else
            savePredMaxValue = predMaxValue;
        if (ex.getDecisionAttribute().compareValues(savePredMinValue, savePredMaxValue) >= 0) {
            ex.setPredictedValue(attr, savePredMaxValue, savePredMinValue, getPredictionLog(predMinRuleId, predMaxRuleId));
        } else {
            ex.setPredictedValue(attr, savePredMinValue, savePredMaxValue, getPredictionLog(predMinRuleId, predMaxRuleId));
        }
    }

    private String getPredictionLog(Long predMinRuleId, Long predMaxRuleId) {
        if (predMinRuleId == null && predMaxRuleId == null) {
            return "";
        } else if (predMinRuleId == null) {
            return String.format("(%d)", predMaxRuleId);
        } else if (predMaxRuleId == null) {
            return String.format("(%d)", predMinRuleId);
        } else {
            return String.format("(%d,%d)", predMinRuleId, predMaxRuleId);
        }
    }
}
