/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dbrsaaorc;

import data.ISF;
import dbrsaaorc.params.Classification;
import dbrsaaorc.params.Coalitions;
import dbrsaaorc.params.Induction;
import dbrsaaorc.params.Latex;
import dbrsaaorc.params.Outranking;
import dbrsaaorc.params.RuleCoalitions;
import domlem.ListOfRules;
import gui.DbrsaaorcJFrame;
import java.io.IOException;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import utils.Lang;
import utils.LatexHelper;

/**
 * Dominance Based Rough Set Approach And Outranking Relation Methods Combine
 *
 * @author Mateusz Ledzianowski
 */
public class Main {

    private static Outranking outranking = Outranking.INDIVIDUAL;
    private static Coalitions coalitions = Coalitions.NONOUTRANKING;
    private static Induction induction = Induction.POSSIBLE;
    private static Classification classification = Classification.NEW;
    private static RuleCoalitions ruleCoalitions = RuleCoalitions.ALL;
    private static Latex latex = Latex.NONE;
    private static ISF dbModel;
    private static ISF dbClassification;
    private static Solver solver;
    private static SolverResult sr;
    private static Classifier classifier;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length > 0 && !args[0].startsWith("-")) {
            initializeSolver(args);
            if (latex != Latex.NONE) {
                System.out.println(args[0] + ": " + solver.getClassification() + " " + solver.getCoalitions() + " " + solver.getInduction() + " " + solver.getOutranking() + " " + solver.getRuleCoalitions() + "\\\\");
            } else {
                System.out.println(args[0] + ": " + solver.getClassification() + " " + solver.getCoalitions() + " " + solver.getInduction() + " " + solver.getOutranking() + " " + solver.getRuleCoalitions());
            }
            calculateAll(args[0]);
            printResults();
            if (args.length > 1 && !args[1].startsWith("-")) {
                classifyAll(args[1]);
                printClassification();
            }
        } else {
            new Thread(new DbrsaaorcJFrame()).start();
            //System.err.println("Usage: java -jar DBRSAaORC.jar <<file_path>>");
        }
    }

    private static void initializeSolver(String[] args) {
        for (String arg : args) {
            if (arg.startsWith("-")) {
                String argName = arg.substring(1).toLowerCase();
                if (Classification.NEW.toString().toLowerCase().startsWith(argName)) {
                    classification = Classification.NEW;
                } else if (Classification.STANDARD.toString().toLowerCase().startsWith(argName)) {
                    classification = Classification.STANDARD;
                } else if (Coalitions.NONOUTRANKING.toString().toLowerCase().startsWith(argName)) {
                    coalitions = Coalitions.NONOUTRANKING;
                } else if (Coalitions.PREFERENCE.toString().toLowerCase().startsWith(argName)) {
                    coalitions = Coalitions.PREFERENCE;
                } else if (Induction.POSSIBLE.toString().toLowerCase().startsWith(argName)) {
                    induction = Induction.POSSIBLE;
                } else if (Induction.CERTAIN.toString().toLowerCase().startsWith(argName)) {
                    induction = Induction.CERTAIN;
                } else if (Outranking.INDIVIDUAL.toString().toLowerCase().startsWith(argName)) {
                    outranking = Outranking.INDIVIDUAL;
                } else if (Outranking.TOGETHER.toString().toLowerCase().startsWith(argName)) {
                    outranking = Outranking.TOGETHER;
                } else if (RuleCoalitions.ALL.toString().toLowerCase().startsWith(argName)) {
                    ruleCoalitions = RuleCoalitions.ALL;
                } else if (RuleCoalitions.FIRST.toString().toLowerCase().startsWith(argName)) {
                    ruleCoalitions = RuleCoalitions.FIRST;
                } else if (RuleCoalitions.RANDOM.toString().toLowerCase().startsWith(argName)) {
                    ruleCoalitions = RuleCoalitions.RANDOM;
                } else if ("latex".startsWith(argName)) {
                    latex = Latex.ANG;
                } else if ("latexpl".startsWith(argName)) {
                    latex = Latex.PL;
                }
            }
        }
        solver = new Solver(classification, coalitions, induction, outranking, ruleCoalitions);
    }

    private static SolverResult calculateAll(String filePath) {
        try {
            dbModel = new ISF(filePath);
            sr = solver.solve(dbModel);
            return sr;
        } catch (IOException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
    }

    private static SolverResult classifyAll(String filePath) {
        try {
            dbClassification = new ISF(filePath);
            Map<String, ListOfRules> positiveRules;
            Map<String, ListOfRules> negativeRules;
            if (induction == Induction.CERTAIN) {
                positiveRules = sr.getPositiveCertainRulesMSCUP();
                negativeRules = sr.getNegativeCertainRulesMSCLP();
            } else {
                positiveRules = sr.getPositivePossibleRulesMSCUP();
                negativeRules = sr.getNegativePossibleRulesMSCLP();
            }
            if (classification == Classification.STANDARD) {
                classifier = new ClassifierStandard(positiveRules, negativeRules, dbClassification.getExamples());
            } else {
                classifier = new ClassifierNew(positiveRules, negativeRules, dbModel, sr.getUpperApproxOfPositiveClassesUnions(), sr.getUpperApproxOfNegativeClassesUnions(), dbClassification.getExamples());
            }
            classifier.classify(latex);
            return sr;
        } catch (IOException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
    }

    private static String labelSuff() {
        return String.format("%s-%s-%s-%s-%s-%s",
                dbModel.getName().toLowerCase(),
                classification.toString().substring(0, 2).toLowerCase(),
                coalitions.toString().substring(0, 2).toLowerCase(),
                induction.toString().substring(0, 2).toLowerCase(),
                outranking.toString().substring(0, 2).toLowerCase(),
                ruleCoalitions.toString().substring(0, 2).toLowerCase());
    }

    private static void printResults() {
        StringBuilder sb = new StringBuilder();
        sb.append(dbModel.toString(latex, labelSuff()));
        if (latex != Latex.NONE) {
            sb.append("\n");
        } else {
            sb.append("\nApproximations of unions:\n");
        }
        sb.append(sr.getApproximationsOfClassesUnionsString(latex, labelSuff()));
        sb.append(sr.getMaximiumInsufficientCoalitionsString(outranking, latex, labelSuff()));
        sb.append(sr.getMinimumSufficientCoalitionsString(outranking, latex, labelSuff()));
        if (induction == Induction.POSSIBLE) {
            sb.append(sr.getPossibleRulesString(latex, true, labelSuff()));
        } else {
            sb.append(sr.getCertainRulesString(latex, true, labelSuff()));
        }
        if (latex != Latex.NONE) {
            System.out.print(sb.toString());
            //.replace("\n", "\\\\\n").replace("\\\\\n\\\\", "\n"));
        } else {
            System.out.print(sb.toString());
        }
    }

    private static void printClassification() {
        StringBuilder sb = new StringBuilder();
        sb.append(classifier.toString(latex, labelSuff()));
        if (latex != Latex.NONE) {
            sb.append(String.format("\n" + Lang.averageError(latex) + ": %s\\\\", LatexHelper.equation(
                    String.format("%.2f", classifier.getAverageError()), latex)));
        } else {
            sb.append(String.format("\nAverage error: %s",
                    String.format("%.2f", classifier.getAverageError())));
        }
        if (latex != Latex.NONE) {
            sb.append(String.format("\n" + Lang.accuracyRatio(latex) + ": %s", LatexHelper.equation(
                    String.format("%.2f\\%%", classifier.getAccuracyRatio() * 100), latex)));
        } else {
            sb.append(String.format("\nAccuracy ratio: %s",
                    String.format("%.2f%%", classifier.getAccuracyRatio() * 100)));
        }
        if (latex != Latex.NONE) {
            System.out.println(sb.toString());
            //.replace("\n", "\\\\\n").replace("\\\\\n\\\\", "\n"));
        } else {
            System.out.println(sb.toString());
        }
    }
}
