/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dbrsaaorc;

import classesUnion.LowerApproxOfNegativeClassesUnion;
import classesUnion.LowerApproxOfPositiveClassesUnion;
import classesUnion.UpperApproxOfNegativeClassesUnion;
import classesUnion.UpperApproxOfPositiveClassesUnion;
import coalition.Coalition;
import coalition.InsufficientCoalitions;
import coalition.MaximumInsufficientCoalitions;
import coalition.MinimumSufficientCoalitions;
import coalition.SetOfCoalitions;
import coalition.SufficientCoalitions;
import core.SetOfExamples;
import data.Attribute;
import data.ISF;
import dbrsaaorc.params.Classification;
import dbrsaaorc.params.Coalitions;
import dbrsaaorc.params.Induction;
import dbrsaaorc.params.Outranking;
import dbrsaaorc.params.RuleCoalitions;
import domlem.ListOfRules;
import java.util.Random;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class Solver {

    private Classification classification;
    private Coalitions coalitions;
    private Induction induction;
    private Outranking outranking;
    private RuleCoalitions ruleCoalitions;
    private final Random rand = new Random();

    public Solver(Classification classification, Coalitions coalitions, Induction induction, Outranking outranking, RuleCoalitions ruleCoalitions) {
        this.classification = classification;
        this.coalitions = coalitions;
        this.induction = induction;
        this.outranking = outranking;
        this.ruleCoalitions = ruleCoalitions;
    }

    public Classification getClassification() {
        return classification;
    }

    public void setClassification(Classification classification) {
        this.classification = classification;
    }

    public Coalitions getCoalitions() {
        return coalitions;
    }

    public void setCoalitions(Coalitions coalitions) {
        this.coalitions = coalitions;
    }

    public Induction getInduction() {
        return induction;
    }

    public void setInduction(Induction induction) {
        this.induction = induction;
    }

    public Outranking getOutranking() {
        return outranking;
    }

    public void setOutranking(Outranking outranking) {
        this.outranking = outranking;
    }

    public void setRuleCoalitions(RuleCoalitions ruleCoalitions) {
        this.ruleCoalitions = ruleCoalitions;
    }

    public RuleCoalitions getRuleCoalitions() {
        return ruleCoalitions;
    }

    public SolverResult solve(ISF db) {
        SolverResult solverResult = new SolverResult(db.getName(), db.getDecisionAttribute());
        calculateApproximationsOfClassesUnions(db, solverResult);
        calculateCoalitions(db, solverResult);
        calculateRules(db, solverResult);
        return solverResult;
    }

    private void calculateApproximationsOfClassesUnions(ISF db, SolverResult solverResult) {
        Attribute decisionAttribute = db.getDecisionAttribute();

        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String decisionValue = decisionAttribute.getValues().get(i);
            LowerApproxOfNegativeClassesUnion laoncu
                    = new LowerApproxOfNegativeClassesUnion(
                            decisionAttribute, decisionValue,
                            db.getExamples());
            solverResult.getLowerApproxOfNegativeClassesUnions().put(decisionValue, laoncu);
            UpperApproxOfNegativeClassesUnion uaoncu
                    = new UpperApproxOfNegativeClassesUnion(
                            decisionAttribute, decisionValue,
                            db.getExamples());
            solverResult.getUpperApproxOfNegativeClassesUnions().put(decisionValue, uaoncu);
            LowerApproxOfPositiveClassesUnion laopcu
                    = new LowerApproxOfPositiveClassesUnion(
                            decisionAttribute, decisionValue,
                            db.getExamples());
            solverResult.getLowerApproxOfPositiveClassesUnions().put(decisionValue, laopcu);
            UpperApproxOfPositiveClassesUnion uaopcu
                    = new UpperApproxOfPositiveClassesUnion(
                            decisionAttribute, decisionValue,
                            db.getExamples());
            solverResult.getUpperApproxOfPositiveClassesUnions().put(decisionValue, uaopcu);
        }
    }

    private void calculateCoalitions(ISF db, SolverResult solverResult) {
        Attribute decisionAttribute = db.getDecisionAttribute();

        SetOfCoalitions allCoalitions = new SetOfCoalitions(Attribute.getSignificantAttributes(db.getAttributes()));

        if (outranking == Outranking.INDIVIDUAL) {
            for (int i = 1; i < decisionAttribute.getValues().size(); ++i) {
                String upValue = decisionAttribute.getValues().get(i);
                String lowValue = decisionAttribute.getValues().get(i - 1);

                InsufficientCoalitions insuffCoalitionsLowerPositive = new InsufficientCoalitions(
                        solverResult.getLowerApproxOfPositiveClassesUnions().get(upValue),
                        solverResult.getUpperApproxOfNegativeClassesUnions().get(lowValue));
                MaximumInsufficientCoalitions maxInsuffCoalitionsLowerPositive = new MaximumInsufficientCoalitions(
                        insuffCoalitionsLowerPositive);
                solverResult.getInsufficientCoalitionsForClassLowerPositive().put(upValue, insuffCoalitionsLowerPositive);
                solverResult.getMaximiumInsufficientCoalitionsForClassLowerPositive().put(upValue, maxInsuffCoalitionsLowerPositive);

                InsufficientCoalitions insuffCoalitionsUpperPositive = new InsufficientCoalitions(
                        solverResult.getUpperApproxOfPositiveClassesUnions().get(upValue),
                        solverResult.getLowerApproxOfNegativeClassesUnions().get(lowValue));
                MaximumInsufficientCoalitions maxInsuffCoalitionsUpperPositive = new MaximumInsufficientCoalitions(
                        insuffCoalitionsUpperPositive);
                solverResult.getInsufficientCoalitionsForClassUpperPositive().put(upValue, insuffCoalitionsUpperPositive);
                solverResult.getMaximiumInsufficientCoalitionsForClassUpperPositive().put(upValue, maxInsuffCoalitionsUpperPositive);

                SufficientCoalitions suffCoalitionsLowerPositive;
                if (coalitions == Coalitions.PREFERENCE) {
                    suffCoalitionsLowerPositive = new SufficientCoalitions(
                            solverResult.getLowerApproxOfPositiveClassesUnions().get(upValue),
                            solverResult.getUpperApproxOfNegativeClassesUnions().get(lowValue));
                } else {
                    suffCoalitionsLowerPositive = new SufficientCoalitions(allCoalitions);
                }
                MinimumSufficientCoalitions minSuffCoalitionsLowerPositive
                        = new MinimumSufficientCoalitions(suffCoalitionsLowerPositive,
                                maxInsuffCoalitionsLowerPositive, db.getCriteria());

                solverResult.getSufficientCoalitionsForClassLowerPositive().put(upValue, suffCoalitionsLowerPositive);
                solverResult.getMinimumSufficientCoalitionsForClassLowerPositive().put(upValue, minSuffCoalitionsLowerPositive);

                SufficientCoalitions suffCoalitionsUpperPositive;
                if (coalitions == Coalitions.PREFERENCE) {
                    suffCoalitionsUpperPositive = new SufficientCoalitions(
                            solverResult.getUpperApproxOfPositiveClassesUnions().get(upValue),
                            solverResult.getLowerApproxOfNegativeClassesUnions().get(lowValue));
                } else {
                    suffCoalitionsUpperPositive = new SufficientCoalitions(allCoalitions);
                }
                MinimumSufficientCoalitions minSuffCoalitionsUpperPositive
                        = new MinimumSufficientCoalitions(suffCoalitionsUpperPositive,
                                maxInsuffCoalitionsUpperPositive, db.getCriteria());
                solverResult.getSufficientCoalitionsForClassUpperPositive().put(upValue, new SufficientCoalitions(suffCoalitionsUpperPositive));
                solverResult.getMinimumSufficientCoalitionsForClassUpperPositive().put(upValue, minSuffCoalitionsUpperPositive);
            }
        } else {
            InsufficientCoalitions insuffCoalitionsLowerPositive = new InsufficientCoalitions(
                    solverResult.getLowerApproxOfPositiveClassesUnions(),
                    solverResult.getUpperApproxOfNegativeClassesUnions());
            MaximumInsufficientCoalitions maxInsuffCoalitionsLowerPositive = new MaximumInsufficientCoalitions(
                    insuffCoalitionsLowerPositive);
            solverResult.getInsufficientCoalitionsForClassLowerPositive().put("", maxInsuffCoalitionsLowerPositive);
            solverResult.getMaximiumInsufficientCoalitionsForClassLowerPositive().put("", maxInsuffCoalitionsLowerPositive);

            InsufficientCoalitions insuffCoalitionsUpperPositive = new InsufficientCoalitions(
                    solverResult.getUpperApproxOfPositiveClassesUnions(),
                    solverResult.getLowerApproxOfNegativeClassesUnions());
            MaximumInsufficientCoalitions maxInsuffCoalitionsUpperPositive = new MaximumInsufficientCoalitions(
                    insuffCoalitionsUpperPositive);
            solverResult.getInsufficientCoalitionsForClassUpperPositive().put("", maxInsuffCoalitionsUpperPositive);
            solverResult.getMaximiumInsufficientCoalitionsForClassUpperPositive().put("", maxInsuffCoalitionsUpperPositive);

            SufficientCoalitions suffCoalitionsLowerPositive;
            if (coalitions == Coalitions.PREFERENCE) {
                suffCoalitionsLowerPositive = new SufficientCoalitions(
                        solverResult.getLowerApproxOfPositiveClassesUnions(),
                        solverResult.getUpperApproxOfNegativeClassesUnions());
            } else {
                suffCoalitionsLowerPositive = new SufficientCoalitions(allCoalitions);
            }
            MinimumSufficientCoalitions minSuffCoalitionsLowerPositive
                    = new MinimumSufficientCoalitions(suffCoalitionsLowerPositive,
                            maxInsuffCoalitionsLowerPositive, db.getCriteria());
            solverResult.getSufficientCoalitionsForClassLowerPositive().put("", new SufficientCoalitions(suffCoalitionsLowerPositive));
            solverResult.getMinimumSufficientCoalitionsForClassLowerPositive().put("", minSuffCoalitionsLowerPositive);

            SufficientCoalitions suffCoalitionsUpperPositive;
            if (coalitions == Coalitions.PREFERENCE) {
                suffCoalitionsUpperPositive = new SufficientCoalitions(
                        solverResult.getUpperApproxOfPositiveClassesUnions(),
                        solverResult.getLowerApproxOfNegativeClassesUnions());
            } else {
                suffCoalitionsUpperPositive = new SufficientCoalitions(allCoalitions);
            }
            MinimumSufficientCoalitions minSuffCoalitionsUpperPositive
                    = new MinimumSufficientCoalitions(suffCoalitionsUpperPositive,
                            maxInsuffCoalitionsUpperPositive, db.getCriteria());
            solverResult.getSufficientCoalitionsForClassUpperPositive().put("", new SufficientCoalitions(suffCoalitionsUpperPositive));
            solverResult.getMinimumSufficientCoalitionsForClassUpperPositive().put("", minSuffCoalitionsUpperPositive);
        }
    }

    private void calculateRules(ISF db, SolverResult solverResult) {
        SetOfCoalitions minSufCoalitions;
        for (int i = 1; i < db.getDecisionAttribute().getValues().size(); ++i) {
            String upValue = db.getDecisionAttribute().getValues().get(i);
            String lowValue = db.getDecisionAttribute().getValues().get(i - 1);
            String minSuffValue = upValue;
            if (outranking == Outranking.TOGETHER) {
                minSuffValue = "";
            }

            minSufCoalitions = solverResult.getMinimumSufficientCoalitionsForClassUpperPositive().get(minSuffValue);
            if (ruleCoalitions == RuleCoalitions.RANDOM) {
                minSufCoalitions = new SetOfCoalitions(getRandomFromCoalitions(minSufCoalitions));
            } else if (ruleCoalitions == RuleCoalitions.FIRST) {
                minSufCoalitions = new SetOfCoalitions(getFirstFromCoalitions(minSufCoalitions));
            }
            ListOfRules negCertainRulesMSCLP = new ListOfRules(
                    new SetOfExamples(db.getExamples()), db.getCriteria(),
                    solverResult.getLowerApproxOfNegativeClassesUnions().get(lowValue),
                    minSufCoalitions, false);
            solverResult.getNegativeCertainRulesMSCLP().put(lowValue, negCertainRulesMSCLP);
            minSufCoalitions = solverResult.getMinimumSufficientCoalitionsForClassLowerPositive().get(minSuffValue);
            if (ruleCoalitions == RuleCoalitions.RANDOM) {
                minSufCoalitions = new SetOfCoalitions(getRandomFromCoalitions(minSufCoalitions));
            } else if (ruleCoalitions == RuleCoalitions.FIRST) {
                minSufCoalitions = new SetOfCoalitions(getFirstFromCoalitions(minSufCoalitions));
            }
            ListOfRules posCertainRulesMSCUP = new ListOfRules(
                    new SetOfExamples(db.getExamples()), db.getCriteria(),
                    solverResult.getLowerApproxOfPositiveClassesUnions().get(upValue),
                    minSufCoalitions);
            solverResult.getPositiveCertainRulesMSCUP().put(upValue, posCertainRulesMSCUP);
        }

        for (int i = 1; i < db.getDecisionAttribute().getValues().size(); ++i) {
            String upValue = db.getDecisionAttribute().getValues().get(i);
            String lowValue = db.getDecisionAttribute().getValues().get(i - 1);
            String minSuffValue = upValue;
            if (outranking == Outranking.TOGETHER) {
                minSuffValue = "";
            }

            minSufCoalitions = solverResult.getMinimumSufficientCoalitionsForClassLowerPositive().get(minSuffValue);
            if (ruleCoalitions == RuleCoalitions.RANDOM) {
                minSufCoalitions = new SetOfCoalitions(getRandomFromCoalitions(minSufCoalitions));
            } else if (ruleCoalitions == RuleCoalitions.FIRST) {
                minSufCoalitions = new SetOfCoalitions(getFirstFromCoalitions(minSufCoalitions));
            }
            ListOfRules negPossibleRulesMSCLP = new ListOfRules(
                    new SetOfExamples(db.getExamples()), db.getCriteria(),
                    solverResult.getUpperApproxOfNegativeClassesUnions().get(lowValue),
                    minSufCoalitions, false);
            solverResult.getNegativePossibleRulesMSCLP().put(lowValue, negPossibleRulesMSCLP);
            minSufCoalitions = solverResult.getMinimumSufficientCoalitionsForClassUpperPositive().get(minSuffValue);
            if (ruleCoalitions == RuleCoalitions.RANDOM) {
                minSufCoalitions = new SetOfCoalitions(getRandomFromCoalitions(minSufCoalitions));
            } else if (ruleCoalitions == RuleCoalitions.FIRST) {
                minSufCoalitions = new SetOfCoalitions(getFirstFromCoalitions(minSufCoalitions));
            }
            ListOfRules posPossibleRulesMSCUP = new ListOfRules(
                    new SetOfExamples(db.getExamples()), db.getCriteria(),
                    solverResult.getUpperApproxOfPositiveClassesUnions().get(upValue),
                    minSufCoalitions);
            solverResult.getPositivePossibleRulesMSCUP().put(upValue, posPossibleRulesMSCUP);
        }
    }

    private SetOfCoalitions getFirstFromCoalitions(SetOfCoalitions coalitions) {
        SetOfCoalitions newCoalitions = new SetOfCoalitions();
        for (Coalition coalition : coalitions) {
            newCoalitions.add(coalition);
            return newCoalitions;
        }
        return newCoalitions;
    }

    private SetOfCoalitions getRandomFromCoalitions(SetOfCoalitions coalitions) {
        SetOfCoalitions newCoalitions = new SetOfCoalitions();
        int size = coalitions.size();
        int item = rand.nextInt(size); // In real life, the Random object should be rather more shared than this
        int i = 0;
        for (Coalition coalition : coalitions) {
            if (i == item) {
                newCoalitions.add(coalition);
                break;
            }
            i++;
        }
        return newCoalitions;
    }
}
