/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dbrsaaorc;

import classesUnion.ClassesUnion;
import coalition.InsufficientCoalitions;
import coalition.MaximumInsufficientCoalitions;
import coalition.MinimumSufficientCoalitions;
import coalition.SufficientCoalitions;
import data.Attribute;
import dbrsaaorc.params.Latex;
import dbrsaaorc.params.Outranking;
import domlem.ListOfRules;
import java.util.HashMap;
import java.util.Map;
import utils.Lang;
import utils.LatexHelper;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class SolverResult {

    protected final String name;
    protected final Attribute decisionAttribute;

    protected final Map<String, ClassesUnion> lowerApproxOfNegativeClassesUnions;
    protected final Map<String, ClassesUnion> upperApproxOfNegativeClassesUnions;
    protected final Map<String, ClassesUnion> lowerApproxOfPositiveClassesUnions;
    protected final Map<String, ClassesUnion> upperApproxOfPositiveClassesUnions;

    protected final Map<String, InsufficientCoalitions> insufficientCoalitionsForClassUpperPositive;
    protected final Map<String, SufficientCoalitions> sufficientCoalitionsForClassUpperPositive;
    protected final Map<String, MaximumInsufficientCoalitions> maximiumInsufficientCoalitionsForClassUpperPositive;
    protected final Map<String, MinimumSufficientCoalitions> minimumSufficientCoalitionsForClassUpperPositive;

    protected final Map<String, InsufficientCoalitions> insufficientCoalitionsForClassLowerPositive;
    protected final Map<String, SufficientCoalitions> sufficientCoalitionsForClassLowerPositive;
    protected final Map<String, MaximumInsufficientCoalitions> maximiumInsufficientCoalitionsForClassLowerPositive;
    protected final Map<String, MinimumSufficientCoalitions> minimumSufficientCoalitionsForClassLowerPositive;

    protected final Map<String, ListOfRules> positiveCertainRules;
    protected final Map<String, ListOfRules> positivePossibleRules;

    protected final Map<String, ListOfRules> negativeCertainRules;
    protected final Map<String, ListOfRules> negativePossibleRules;

    public SolverResult(String name, Attribute decisionAttribute) {
        this.name = name;
        this.decisionAttribute = decisionAttribute;

        lowerApproxOfNegativeClassesUnions = new HashMap<>();
        upperApproxOfPositiveClassesUnions = new HashMap<>();
        lowerApproxOfPositiveClassesUnions = new HashMap<>();
        upperApproxOfNegativeClassesUnions = new HashMap<>();

        insufficientCoalitionsForClassUpperPositive = new HashMap<>();
        sufficientCoalitionsForClassUpperPositive = new HashMap<>();
        maximiumInsufficientCoalitionsForClassUpperPositive = new HashMap<>();
        minimumSufficientCoalitionsForClassUpperPositive = new HashMap<>();

        insufficientCoalitionsForClassLowerPositive = new HashMap<>();
        sufficientCoalitionsForClassLowerPositive = new HashMap<>();
        maximiumInsufficientCoalitionsForClassLowerPositive = new HashMap<>();
        minimumSufficientCoalitionsForClassLowerPositive = new HashMap<>();

        positiveCertainRules = new HashMap<>();
        positivePossibleRules = new HashMap<>();

        negativeCertainRules = new HashMap<>();
        negativePossibleRules = new HashMap<>();
    }

    public Attribute getDecisionAttribute() {
        return decisionAttribute;
    }

    public Map<String, ClassesUnion> getLowerApproxOfNegativeClassesUnions() {
        return lowerApproxOfNegativeClassesUnions;
    }

    public String getApproximationsOfClassesUnionsString(Latex latex, String labelSuff) {
        StringBuilder sb = new StringBuilder();
        if (latex != Latex.NONE) {
            sb.append("\\begin{table}[H]\n"
                    + "\\centering\n"
                    + "\\begin{tabularx}{\\textwidth}{@{}lXX@{}}\n"
                    + "\\toprule\n"
                    + Lang.union(latex) + " & "
                    + Lang.lowerApprox(latex) + " & "
                    + Lang.upperApprox(latex) + " \\\\ \\midrule\n");
            for (int i = 0; i < decisionAttribute.getValues().size() - 1; ++i) {
                String attrVal = decisionAttribute.getValues().get(i);
                if (lowerApproxOfNegativeClassesUnions.containsKey(attrVal)
                        || upperApproxOfNegativeClassesUnions.containsKey(attrVal)) {
                    sb.append(LatexHelper.equation(
                            String.format("Cl_{%s}^{\\leq}", attrVal), latex));
                    sb.append(" & ");
                    sb.append(LatexHelper.equation(
                            lowerApproxOfNegativeClassesUnions.get(attrVal).toString(latex),
                            latex));
                    sb.append(" & ");
                    sb.append(LatexHelper.equation(
                            upperApproxOfNegativeClassesUnions.get(attrVal).toString(latex),
                            latex));
                    sb.append(" \\\\\n");
                }
            }
            for (int i = 1; i < decisionAttribute.getValues().size(); ++i) {
                String attrVal = decisionAttribute.getValues().get(i);
                if (lowerApproxOfPositiveClassesUnions.containsKey(attrVal)
                        || upperApproxOfPositiveClassesUnions.containsKey(attrVal)) {
                    sb.append(LatexHelper.equation(
                            String.format("Cl_{%s}^{\\geq}", attrVal), latex));
                    sb.append(" & ");
                    sb.append(LatexHelper.equation(
                            lowerApproxOfPositiveClassesUnions.get(attrVal).toString(latex),
                            latex));
                    sb.append(" & ");
                    sb.append(LatexHelper.equation(
                            upperApproxOfPositiveClassesUnions.get(attrVal).toString(latex),
                            latex));
                    sb.append(" \\\\\n");
                }
            }
            sb.setLength(sb.length() - 1);
            sb.append(" \\bottomrule\n"
                    + "\\end{tabularx}\n"
                    + "\\caption{" + Lang.approxCaption(latex)
                    + " \\textit{" + name + "}.}\n"
                    + "\\label{approx-" + labelSuff + "}\n"
                    + "\\end{table}\n\n");
        } else {
            sb.append(getLowerApproxOfNegativeClassesUnionsString(latex));
            sb.append("\n\n");
            sb.append(getUpperApproxOfNegativeClassesUnionsString(latex));
            sb.append("\n\n");
            sb.append(getLowerApproxOfPositiveClassesUnionsString(latex));
            sb.append("\n\n");
            sb.append(getUpperApproxOfPositiveClassesUnionsString(latex));
            sb.append("\n\n");
        }
        return sb.toString();
    }

    public String getLowerApproxOfNegativeClassesUnionsString(Latex latex) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size() - 1; ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (lowerApproxOfNegativeClassesUnions.containsKey(attrVal)) {
                sb.append(lowerApproxOfNegativeClassesUnions.get(attrVal).toString(latex));
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getLowerApproxOfNegativeClassesUnionsString() {
        return getLowerApproxOfNegativeClassesUnionsString(Latex.NONE);
    }

    public Map<String, ClassesUnion> getUpperApproxOfNegativeClassesUnions() {
        return upperApproxOfNegativeClassesUnions;
    }

    public String getUpperApproxOfNegativeClassesUnionsString(Latex latex) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size() - 1; ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (upperApproxOfNegativeClassesUnions.containsKey(attrVal)) {
                sb.append(upperApproxOfNegativeClassesUnions.get(attrVal).toString(latex));
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getUpperApproxOfNegativeClassesUnionsString() {
        return getUpperApproxOfNegativeClassesUnionsString(Latex.NONE);
    }

    public Map<String, ClassesUnion> getLowerApproxOfPositiveClassesUnions() {
        return lowerApproxOfPositiveClassesUnions;
    }

    public String getLowerApproxOfPositiveClassesUnionsString(Latex latex) {
        StringBuilder sb = new StringBuilder();
        for (int i = 1; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (lowerApproxOfPositiveClassesUnions.containsKey(attrVal)) {
                sb.append(lowerApproxOfPositiveClassesUnions.get(attrVal).toString(latex));
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getLowerApproxOfPositiveClassesUnionsString() {
        return getLowerApproxOfPositiveClassesUnionsString(Latex.NONE);
    }

    public Map<String, ClassesUnion> getUpperApproxOfPositiveClassesUnions() {
        return upperApproxOfPositiveClassesUnions;
    }

    public String getUpperApproxOfPositiveClassesUnionsString(Latex latex) {
        StringBuilder sb = new StringBuilder();
        for (int i = 1; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (upperApproxOfPositiveClassesUnions.containsKey(attrVal)) {
                sb.append(upperApproxOfPositiveClassesUnions.get(attrVal).toString(latex));
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getUpperApproxOfPositiveClassesUnionsString() {
        return getUpperApproxOfPositiveClassesUnionsString(Latex.NONE);
    }

    public Map<String, InsufficientCoalitions> getInsufficientCoalitionsForClassUpperPositive() {
        return insufficientCoalitionsForClassUpperPositive;
    }

    public String getMaximiumInsufficientCoalitionsString(Outranking outranking, Latex latex, String labelSuff) {
        StringBuilder sb = new StringBuilder();
        if (latex != Latex.NONE) {
            sb.append("\\begin{table}[H]\n");
            sb.append("\\centering\n");
            if (outranking == Outranking.TOGETHER) {
                sb.append("\\begin{tabularx}{\\textwidth}{@{}XX@{}}\n");
            } else {
                sb.append("\\begin{tabularx}{\\textwidth}{@{}XXX@{}}\n");
            }
            sb.append("\\toprule\n");
            if (outranking != Outranking.TOGETHER) {
                sb.append("$Cl_{t}^{\\geq}$ & ");
            }
            sb.append("$\\overline{P}(Cl_{t-1}^{\\leq})$ i $\\underline{P}(Cl_{t}^{\\geq})$ & "
                    + "$\\underline{P}(Cl_{t-1}^{\\leq})$ i $\\overline{P}(Cl_{t}^{\\geq})$ "
                    + "\\\\ \\midrule\n");
            if (outranking == Outranking.TOGETHER) {
                if (maximiumInsufficientCoalitionsForClassLowerPositive.containsKey("")) {
                    sb.append(LatexHelper.equation(
                            maximiumInsufficientCoalitionsForClassLowerPositive
                                    .get("").toString(latex), latex));
                }
                sb.append(" & ");
                if (maximiumInsufficientCoalitionsForClassUpperPositive.containsKey("")) {
                    sb.append(LatexHelper.equation(
                            maximiumInsufficientCoalitionsForClassUpperPositive
                                    .get("").toString(latex), latex));
                }
                sb.append(" \\\\\n");
            } else {
                for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
                    String attrVal = decisionAttribute.getValues().get(i);
                    if (maximiumInsufficientCoalitionsForClassLowerPositive.containsKey(attrVal)
                            || maximiumInsufficientCoalitionsForClassUpperPositive.containsKey(attrVal)) {
                        sb.append(LatexHelper.equation(String.format("Cl_%s", attrVal), latex));
                        sb.append(" & ");
                        sb.append(LatexHelper.equation(
                                maximiumInsufficientCoalitionsForClassLowerPositive
                                        .get(attrVal).toString(latex), latex));
                        sb.append(" & ");
                        sb.append(LatexHelper.equation(
                                maximiumInsufficientCoalitionsForClassUpperPositive
                                        .get(attrVal).toString(latex), latex));
                        sb.append(" \\\\\n");
                    }
                }
            }
            sb.setLength(sb.length() - 1);
            sb.append(" \\bottomrule\n"
                    + "\\end{tabularx}\n"
                    + "\\caption{" + Lang.insuffCoalsCaption(latex)
                    + " \\textit{" + name + "}.}\n"
                    + "\\label{maxinsuf-" + labelSuff + "}\n"
                    + "\\end{table}\n\n");
        } else {
            sb.append("Maximum insufficient coalitions of criteria for lower positive:\n");
            sb.append(getMaximiumInsufficientCoalitionsForClassLowerPositiveString(outranking, latex));
            sb.append("\n\nMaximum insufficient coalitions of criteria for upper positive:\n");
            sb.append(getMaximiumInsufficientCoalitionsForClassUpperPositiveString(outranking, latex));
            sb.append("\n\n");
        }
        return sb.toString();
    }

    public String getMinimumSufficientCoalitionsString(Outranking outranking, Latex latex, String labelSuff) {
        StringBuilder sb = new StringBuilder();
        if (latex != Latex.NONE) {
            sb.append("\\begin{table}[H]\n"
                    + "\\centering\n"
                    + "\\begin{tabularx}{\\textwidth}{@{}XXX@{}}\n"
                    + "\\toprule\n");
            if (outranking != Outranking.TOGETHER) {
                sb.append("$Cl_{t}^{\\geq}$ & ");
            }
            sb.append("$\\overline{P}(Cl_{t-1}^{\\leq})$ i $\\underline{P}(Cl_{t}^{\\geq})$ & "
                    + "$\\underline{P}(Cl_{t-1}^{\\leq})$ i $\\overline{P}(Cl_{t}^{\\geq})$ "
                    + "\\\\ \\midrule\n");
            if (outranking == Outranking.TOGETHER) {
                if (minimumSufficientCoalitionsForClassLowerPositive.containsKey("")) {
                    sb.append(LatexHelper.equation(
                            minimumSufficientCoalitionsForClassLowerPositive
                                    .get("").toString(latex), latex));
                }
                sb.append(" & ");
                if (minimumSufficientCoalitionsForClassUpperPositive.containsKey("")) {
                    sb.append(LatexHelper.equation(
                            minimumSufficientCoalitionsForClassUpperPositive
                                    .get("").toString(latex), latex));
                }
                sb.append(" \\\\\n");
            } else {
                for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
                    String attrVal = decisionAttribute.getValues().get(i);
                    if (minimumSufficientCoalitionsForClassLowerPositive.containsKey(attrVal)
                            || minimumSufficientCoalitionsForClassUpperPositive.containsKey(attrVal)) {
                        sb.append(LatexHelper.equation(String.format("Cl_%s", attrVal), latex));
                        sb.append(" & ");
                        sb.append(LatexHelper.equation(
                                minimumSufficientCoalitionsForClassLowerPositive
                                        .get(attrVal).toString(latex), latex));
                        sb.append(" & ");
                        sb.append(LatexHelper.equation(
                                minimumSufficientCoalitionsForClassUpperPositive
                                        .get(attrVal).toString(latex), latex));
                        sb.append(" \\\\\n");
                    }
                }
            }
            sb.setLength(sb.length() - 1);
            sb.append(" \\bottomrule\n"
                    + "\\end{tabularx}\n"
                    + "\\caption{" + Lang.suffCoalsCaption(latex)
                    + " \\textit{" + name + "}.}\n"
                    + "\\label{minsuf-" + labelSuff + "}\n"
                    + "\\end{table}\n\n");
        } else {
            sb.append("Minimum sufficient coalitions of criteria for lower positive:\n");
            sb.append(getMinimumSufficientCoalitionsForClassLowerPositiveString(outranking, latex));
            sb.append("\n\nMinimum sufficient coalitions of criteria for upper positive:\n");
            sb.append(getMinimumSufficientCoalitionsForClassUpperPositiveString(outranking, latex));
            sb.append("\n\n");
        }
        return sb.toString();
    }

    public String getInsufficientCoalitionsForClassUpperPositiveString(Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (insufficientCoalitionsForClassUpperPositive.containsKey("")) {
                return LatexHelper.equation(
                        insufficientCoalitionsForClassUpperPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (insufficientCoalitionsForClassUpperPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            insufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            insufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getInsufficientCoalitionsForClassUpperPositiveString(Outranking outranking) {
        return getInsufficientCoalitionsForClassUpperPositiveString(outranking, Latex.NONE);
    }

    public Map<String, SufficientCoalitions> getSufficientCoalitionsForClassUpperPositive() {
        return sufficientCoalitionsForClassUpperPositive;
    }

    public String getSufficientCoalitionsForClassUpperPositiveString(Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (sufficientCoalitionsForClassUpperPositive.containsKey("")) {
                return LatexHelper.equation(
                        sufficientCoalitionsForClassUpperPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (sufficientCoalitionsForClassUpperPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            sufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            sufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getSufficientCoalitionsForClassUpperPositiveString(Outranking outranking) {
        return getSufficientCoalitionsForClassUpperPositiveString(outranking, Latex.NONE);
    }

    public Map<String, MaximumInsufficientCoalitions> getMaximiumInsufficientCoalitionsForClassUpperPositive() {
        return maximiumInsufficientCoalitionsForClassUpperPositive;
    }

    public String getMaximiumInsufficientCoalitionsForClassUpperPositiveString(Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (maximiumInsufficientCoalitionsForClassUpperPositive.containsKey("")) {
                return LatexHelper.equation(
                        maximiumInsufficientCoalitionsForClassUpperPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (maximiumInsufficientCoalitionsForClassUpperPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            maximiumInsufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            maximiumInsufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getMaximiumInsufficientCoalitionsForClassUpperPositiveString(Outranking outranking) {
        return getMaximiumInsufficientCoalitionsForClassUpperPositiveString(outranking, Latex.NONE);
    }

    public Map<String, MinimumSufficientCoalitions> getMinimumSufficientCoalitionsForClassUpperPositive() {
        return minimumSufficientCoalitionsForClassUpperPositive;
    }

    public String getMinimumSufficientCoalitionsForClassUpperPositiveString(Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (minimumSufficientCoalitionsForClassUpperPositive.containsKey("")) {
                return LatexHelper.equation(
                        minimumSufficientCoalitionsForClassUpperPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (minimumSufficientCoalitionsForClassUpperPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            minimumSufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            minimumSufficientCoalitionsForClassUpperPositive.get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getMinimumSufficientCoalitionsForClassUpperPositiveString(Outranking outranking) {
        return getMinimumSufficientCoalitionsForClassUpperPositiveString(outranking, Latex.NONE);
    }

    public Map<String, InsufficientCoalitions> getInsufficientCoalitionsForClassLowerPositive() {
        return insufficientCoalitionsForClassLowerPositive;
    }

    public String getInsufficientCoalitionsForClassLowerPositiveString(Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (insufficientCoalitionsForClassLowerPositive.containsKey("")) {
                return LatexHelper.equation(
                        insufficientCoalitionsForClassLowerPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (insufficientCoalitionsForClassLowerPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            insufficientCoalitionsForClassLowerPositive.get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            insufficientCoalitionsForClassLowerPositive.get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getInsufficientCoalitionsForClassLowerPositiveString(Outranking outranking) {
        return getInsufficientCoalitionsForClassLowerPositiveString(outranking, Latex.NONE);
    }

    public Map<String, SufficientCoalitions> getSufficientCoalitionsForClassLowerPositive() {
        return sufficientCoalitionsForClassLowerPositive;
    }

    public String getSufficientCoalitionsForClassLowerPositiveString(Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (sufficientCoalitionsForClassLowerPositive.containsKey("")) {
                return LatexHelper.equation(
                        sufficientCoalitionsForClassLowerPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (sufficientCoalitionsForClassLowerPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            sufficientCoalitionsForClassLowerPositive.get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            sufficientCoalitionsForClassLowerPositive.get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getSufficientCoalitionsForClassLowerPositiveString(Outranking outranking) {
        return getSufficientCoalitionsForClassLowerPositiveString(outranking, Latex.NONE);
    }

    public Map<String, MaximumInsufficientCoalitions> getMaximiumInsufficientCoalitionsForClassLowerPositive() {
        return maximiumInsufficientCoalitionsForClassLowerPositive;
    }

    public String getMaximiumInsufficientCoalitionsForClassLowerPositiveString(
            Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (maximiumInsufficientCoalitionsForClassLowerPositive.containsKey("")) {
                return LatexHelper.equation(
                        maximiumInsufficientCoalitionsForClassLowerPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (maximiumInsufficientCoalitionsForClassLowerPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            maximiumInsufficientCoalitionsForClassLowerPositive
                                    .get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            maximiumInsufficientCoalitionsForClassLowerPositive
                                    .get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getMaximiumInsufficientCoalitionsForClassLowerPositiveString(Outranking outranking) {
        return getMaximiumInsufficientCoalitionsForClassLowerPositiveString(outranking, Latex.NONE);
    }

    public Map<String, MinimumSufficientCoalitions> getMinimumSufficientCoalitionsForClassLowerPositive() {
        return minimumSufficientCoalitionsForClassLowerPositive;
    }

    public String getMinimumSufficientCoalitionsForClassLowerPositiveString(Outranking outranking, Latex latex) {
        if (outranking == Outranking.TOGETHER) {
            if (minimumSufficientCoalitionsForClassLowerPositive.containsKey("")) {
                return LatexHelper.equation(
                        minimumSufficientCoalitionsForClassLowerPositive
                                .get("").toString(latex), latex);
            }
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (minimumSufficientCoalitionsForClassLowerPositive.containsKey(attrVal)) {
                if (latex != Latex.NONE) {
                    sb.append(String.format("$Cl_%s: %s$\n", attrVal,
                            minimumSufficientCoalitionsForClassLowerPositive.get(attrVal).toString(latex)));
                } else {
                    sb.append(String.format("Class[%s]: %s\n", attrVal,
                            minimumSufficientCoalitionsForClassLowerPositive.get(attrVal).toString(latex)));
                }
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getMinimumSufficientCoalitionsForClassLowerPositiveString(Outranking outranking) {
        return getMinimumSufficientCoalitionsForClassLowerPositiveString(outranking, Latex.NONE);
    }

    String getPossibleRulesString(Latex latex, boolean sortedConditions, String labelSuff) {
        StringBuilder sb = new StringBuilder();
        if (latex != Latex.NONE) {
            sb.append("\\begin{table}[H]\n"
                    + "\\centering\n"
                    + "\\begin{tabularx}{\\textwidth}{@{}lllX@{}}\n"
                    + "\\toprule\n"
                    + Lang.number(latex) + " & "
                    + Lang.conditionPart(latex) + " & "
                    + Lang.decision(latex) + " & "
                    + Lang.support(latex) + " \\\\ \\midrule\n");
            for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
                String attrVal = decisionAttribute.getValues().get(i);
                if (negativePossibleRules.containsKey(attrVal)) {
                    sb.append(negativePossibleRules.get(attrVal).toString(latex, sortedConditions));
                }
            }
            for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
                String attrVal = decisionAttribute.getValues().get(i);
                if (positivePossibleRules.containsKey(attrVal)) {
                    sb.append(positivePossibleRules.get(attrVal).toString(latex, sortedConditions));
                }
            }
            sb.setLength(sb.length() - 1);
            sb.append(" \\bottomrule\n"
                    + "\\end{tabularx}\n"
                    + "\\caption{" + Lang.possibleRulesCaption(latex)
                    + " \\textit{" + name + "}.}\n"
                    + "\\label{possible-" + labelSuff + "}\n"
                    + "\\end{table}\n\n");
        } else {
            sb.append("Possible rules:\n");
            sb.append(getNegativePossibleRulesString(latex, sortedConditions));
            sb.append("\n");
            sb.append(getPositivePossibleRulesString(latex, sortedConditions));
            sb.append("\n");
        }
        return sb.toString();
    }

    String getCertainRulesString(Latex latex, boolean sortedConditions, String labelSuff) {
        StringBuilder sb = new StringBuilder();
        if (latex != Latex.NONE) {
            sb.append("\\begin{table}[H]\n"
                    + "\\centering\n"
                    + "\\begin{tabularx}{\\textwidth}{@{}lllX@{}}\n"
                    + "\\toprule\n"
                    + Lang.number(latex) + " & "
                    + Lang.conditionPart(latex) + " & "
                    + Lang.decision(latex) + " & "
                    + Lang.support(latex) + " \\\\ \\midrule\n");
            for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
                String attrVal = decisionAttribute.getValues().get(i);
                if (negativeCertainRules.containsKey(attrVal)) {
                    sb.append(negativeCertainRules.get(attrVal).toString(latex, sortedConditions));
                }
            }
            for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
                String attrVal = decisionAttribute.getValues().get(i);
                if (positiveCertainRules.containsKey(attrVal)) {
                    sb.append(positiveCertainRules.get(attrVal).toString(latex, sortedConditions));
                }
            }
            sb.setLength(sb.length() - 1);
            sb.append(" \\bottomrule\n"
                    + "\\end{tabularx}\n"
                    + "\\caption{" + Lang.possibleRulesCaption(latex)
                    + " \\textit{" + name + "}.}\n"
                    + "\\label{certain-" + labelSuff + "}\n"
                    + "\\end{table}\n\n");
        } else {
            sb.append("Certain rules:\n");
            sb.append(getNegativeCertainRulesString(latex, sortedConditions));
            sb.append("\n");
            sb.append(getPositiveCertainRulesString(latex, sortedConditions));
            sb.append("\n");
        }
        return sb.toString();
    }

    public Map<String, ListOfRules> getPositiveCertainRulesMSCUP() {
        return positiveCertainRules;
    }

    public String getPositiveCertainRulesString(Latex latex, boolean sortedConditions) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (positiveCertainRules.containsKey(attrVal)) {
                sb.append(positiveCertainRules.get(attrVal).toString(latex, sortedConditions));
                if (latex != Latex.NONE) {
                    sb.setLength(sb.length() - 3);
                    sb.append("\n");
                }
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getPositiveCertainRulesMSCUPString() {
        return getPositiveCertainRulesString(Latex.NONE, true);
    }

    public Map<String, ListOfRules> getPositivePossibleRulesMSCUP() {
        return positivePossibleRules;
    }

    public String getPositivePossibleRulesString(Latex latex, boolean sortedConditions) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (positivePossibleRules.containsKey(attrVal)) {
                sb.append(positivePossibleRules.get(attrVal).toString(latex, sortedConditions));
                if (latex != Latex.NONE) {
                    sb.setLength(sb.length() - 3);
                    sb.append("\n");
                }
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getPositivePossibleRulesMSCUPString() {
        return getPositivePossibleRulesString(Latex.NONE, true);
    }

    public Map<String, ListOfRules> getNegativeCertainRulesMSCLP() {
        return negativeCertainRules;
    }

    public String getNegativeCertainRulesString(Latex latex, boolean sortedConditions) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (negativeCertainRules.containsKey(attrVal)) {
                sb.append(negativeCertainRules.get(attrVal).toString(latex, sortedConditions));
                if (latex != Latex.NONE) {
                    sb.setLength(sb.length() - 3);
                    sb.append("\n");
                }
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getNegativeCertainRulesMSCLPString() {
        return getNegativeCertainRulesString(Latex.NONE, true);
    }

    public Map<String, ListOfRules> getNegativePossibleRulesMSCLP() {
        return negativePossibleRules;
    }

    public String getNegativePossibleRulesString(Latex latex, boolean sortedConditions) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < decisionAttribute.getValues().size(); ++i) {
            String attrVal = decisionAttribute.getValues().get(i);
            if (negativePossibleRules.containsKey(attrVal)) {
                sb.append(negativePossibleRules.get(attrVal).toString(latex, sortedConditions));
                if (latex != Latex.NONE) {
                    sb.setLength(sb.length() - 3);
                    sb.append("\n");
                }
                sb.append("\n");
            }
        }
        if (sb.length() == 0) {
            return "";
        }
        return sb.substring(0, sb.length() - 1);
    }

    public String getNegativePossibleRulesMSCLPString() {
        return getNegativePossibleRulesString(Latex.NONE, true);
    }
}
