/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dominanceCone;

import data.Example;
import core.SetOfExamples;
import java.util.List;
import java.util.Objects;

/**
 *
 * @author Mateusz Ledzianowski
 */
public abstract class DominanceCone extends SetOfExamples {

    /**
     * A main example of dominance cone. All examples in dominance cone must
     * dominate main example (in positive dominance cone) or be dominated by
     * main example (in negative dominance cone).
     */
    private final Example mainExample;

    /**
     * Basic constructor of the dominance cone
     *
     * @param mainExample
     */
    public DominanceCone(Example mainExample) {
        super();
        this.mainExample = mainExample;
    }

    /**
     * Function calculates dominance cone examples from the given list of
     * examples.
     *
     * @param examples
     */
    protected abstract void calculateDominanceCone(List<Example> examples);

    /**
     * @return hash code of the object
     */
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 23 * hash + Objects.hashCode(this.mainExample);
        return hash;
    }

    /**
     * @param obj object to compare
     * @return true if objects are equal
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final DominanceCone other = (DominanceCone) obj;
        return Objects.equals(this.mainExample, other.mainExample);
    }

    /**
     * @return the mainExample
     */
    public Example getMainExample() {
        return mainExample;
    }
}
