/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package domlem;

import data.Attribute;
import data.AttributePreference;
import data.Example;
import dbrsaaorc.params.Latex;
import java.util.Objects;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class Condition implements Comparable<Condition> {

    private Attribute attribute;
    private String attributeValue;
    /**
     * True if greater or equal, False if less or equal
     */
    private boolean greaterOrEqual;

    public Condition(Attribute attribute, String attributeValue) {
        this.attribute = attribute;
        this.attributeValue = attributeValue;
        this.greaterOrEqual = true;
    }

    public Condition(Attribute attribute, String attributeValue, boolean greaterOrEqual) {
        this.attribute = attribute;
        this.attributeValue = attributeValue;
        this.greaterOrEqual = greaterOrEqual;
    }

    public boolean isFulfilled(Example ex) {
        if (greaterOrEqual) {
            return attribute.compareValues(ex.getAttributeValue(attribute), attributeValue) >= 0;
        }
        return attribute.compareValues(ex.getAttributeValue(attribute), attributeValue) <= 0;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 83 * hash + Objects.hashCode(this.attribute);
        hash = 83 * hash + Objects.hashCode(this.attributeValue);
        hash = 83 * hash + (this.greaterOrEqual ? 1 : 0);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Condition other = (Condition) obj;
        if (this.greaterOrEqual != other.greaterOrEqual) {
            return false;
        }
        if (!Objects.equals(this.attributeValue, other.attributeValue)) {
            return false;
        }
        return Objects.equals(this.attribute, other.attribute);
    }

    public String toString(Latex latex) {
        return attribute + getOperator(latex) + attributeValue;
    }

    @Override
    public String toString() {
        return toString(Latex.NONE);
    }

    public String toShortString(Latex latex) {
        return attribute.getName(latex) + getOperator(latex) + attributeValue;
    }

    public String toShortString() {
        return toShortString(Latex.NONE);
    }

    private String getOperator(Latex latex) {
        if (latex != Latex.NONE) {
            return (greaterOrEqual
                    ? (attribute.getPreference() == AttributePreference.GAIN ? " \\geq " : " \\leq ")
                    : (attribute.getPreference() == AttributePreference.GAIN ? " \\leq " : " \\geq "));
        } else {
            return (greaterOrEqual
                    ? (attribute.getPreference() == AttributePreference.GAIN ? ">=" : "<=")
                    : (attribute.getPreference() == AttributePreference.GAIN ? "<=" : ">="));
        }
    }

    public Attribute getAttribute() {
        return attribute;
    }

    public void setAttribute(Attribute attribute) {
        this.attribute = attribute;
    }

    public String getAttributeValue() {
        return attributeValue;
    }

    public void setAttributeValue(String attributeValue) {
        this.attributeValue = attributeValue;
    }

    public boolean isGreaterOrEqual() {
        return greaterOrEqual;
    }

    public void setGreaterOrEqual(boolean greaterOrEqual) {
        this.greaterOrEqual = greaterOrEqual;
    }

    @Override
    public int compareTo(Condition o) {
        return this.attribute.compareTo(o.attribute);
    }
}
