/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package domlem;

import core.SetOfExamples;
import data.Example;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class ConditionScore {

    /**
     * Number of examples which satisfies both conditions and decision
     */
    private int support;
    /**
     * Support divided by cardinality of all set
     */
    private float strength;
    /**
     * Support divided by number of examples which satisfies conditions
     */
    private float certainty;
    /**
     * Support divided by number of examples which satisfies decision
     */
    private float coverage;
    /**
     * Set of examples that fulfilled condition
     */
    protected final SetOfExamples examplesInCondition;
    /**
     * Set of examples that fulfilled condition and decision
     */
    protected final SetOfExamples examplesInConditionAndDecision;
    /**
     * Set of examples that fulfilled condition and not decision
     */
    protected final SetOfExamples examplesInConditionAndNotDecision;

    public ConditionScore() {
        this.examplesInCondition = new SetOfExamples();
        this.examplesInConditionAndDecision = new SetOfExamples();
        this.examplesInConditionAndNotDecision = new SetOfExamples();
    }

    public ConditionScore(int support, float strength, float certainty, float coverage) {
        this.support = support;
        this.strength = strength;
        this.certainty = certainty;
        this.coverage = coverage;
        this.examplesInCondition = new SetOfExamples();
        this.examplesInConditionAndDecision = new SetOfExamples();
        this.examplesInConditionAndNotDecision = new SetOfExamples();
    }

    public ConditionScore(Condition cond, SetOfExamples examples, Decision decision, String decisionValue) {
        this.examplesInCondition = new SetOfExamples();
        this.examplesInConditionAndDecision = new SetOfExamples();
        this.examplesInConditionAndNotDecision = new SetOfExamples();
        int _support = 0;
        int _nAllExamples = 0;
        int _nSatisfiesDecision = 0;
        int _nSatisfiesCondition = 0;
        for (Example ex : examples) {
            ++_nAllExamples;
            if (decision.isFulfilled(ex)) {
                ++_nSatisfiesDecision;
            }
            if (cond.isFulfilled(ex)) {
                examplesInCondition.add(ex);
                ++_nSatisfiesCondition;
                if (decision.isFulfilled(ex)) {
                    examplesInConditionAndDecision.add(ex);
                    ++_support;
                } else {
                    examplesInConditionAndNotDecision.add(ex);
                }
            }
        }
        this.support = _support;
        this.strength = getStrength(_support, _nAllExamples);
        this.certainty = getCertainty(_support, _nSatisfiesCondition);
        this.coverage = getCoverage(_support, _nSatisfiesDecision);
    }

    public static float getStrength(int support, int nAllExamples) {
        if (nAllExamples == 0) {
            return 0;
        }
        return (float) support / nAllExamples;
    }

    public static float getCertainty(int support, int nFromUnion) {
        if (nFromUnion == 0) {
            return 0;
        }
        return (float) support / nFromUnion;
    }

    public static float getCoverage(int support, int nFromClass) {
        if (nFromClass == 0) {
            return 0;
        }
        return (float) support / nFromClass;
    }

    public int getSupport() {
        return support;
    }

    public float getStrength() {
        return strength;
    }

    public float getCertainty() {
        return certainty;
    }

    public float getCoverage() {
        return coverage;
    }

    protected void setSupport(int support) {
        this.support = support;
    }

    protected void setStrength(float strength) {
        this.strength = strength;
    }

    protected void setCertainty(float certainty) {
        this.certainty = certainty;
    }

    protected void setCoverage(float coverage) {
        this.coverage = coverage;
    }

    public SetOfExamples getExamplesInCondition() {
        return examplesInCondition;
    }

    public SetOfExamples getExamplesInConditionAndDecision() {
        return examplesInConditionAndDecision;
    }

    public SetOfExamples getExamplesInConditionAndNotDecision() {
        return examplesInConditionAndNotDecision;
    }

    @Override
    public String toString() {
        return "sup=" + support + ", str=" + strength + ", cer=" + certainty + ", cov=" + coverage;
    }
}
