/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package domlem;

import classesUnion.ClassesUnion;
import coalition.SetOfCoalitions;
import core.SetOfExamples;
import data.Attribute;
import data.Example;
import dbrsaaorc.params.Latex;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Mateusz Ledzianowski
 */
public class ListOfRules extends ArrayList<Rule> {

    private SetOfCoalitions minimumSufficientCoalitions;

    public ListOfRules() {
        super(new ArrayList<>());
        minimumSufficientCoalitions = new SetOfCoalitions();
    }

    public ListOfRules(ListOfRules listOfRules) {
        super(listOfRules);
        minimumSufficientCoalitions = new SetOfCoalitions();
    }

    public ListOfRules(SetOfExamples examples, List<Attribute> criteria, ClassesUnion classesUnion, SetOfCoalitions minimumSufficientCoalitions) {
        this(examples, criteria, classesUnion, minimumSufficientCoalitions, true);
    }

    public ListOfRules(SetOfExamples examples, List<Attribute> criteria, ClassesUnion classesUnion, SetOfCoalitions minimumSufficientCoalitions, boolean greaterOrEqual) {
        super(new ArrayList<>());
        this.minimumSufficientCoalitions = minimumSufficientCoalitions;
        SetOfExamples examplesToCover = new SetOfExamples(classesUnion); // Examples which remain to cover
        SetOfExamples examplesNotCovered = new SetOfExamples(examples); // Examples which are not covered

        while (!examplesToCover.isEmpty()) {
            Rule rule = new Rule(
                    new ArrayList<>(),
                    new Decision(classesUnion.getDecisionAttribute(),
                            classesUnion.getDecisionValue(), greaterOrEqual));
            while (!rule.isSufficient(minimumSufficientCoalitions)) {
                rule = rule.addBestCondition(minimumSufficientCoalitions, examplesToCover, examplesNotCovered);
            }
            rule = rule.getSimplifiedRule(minimumSufficientCoalitions, examplesToCover, examplesNotCovered);
            rule.setRuleScore(examples);
            super.add(rule);

            for (Example ex : examples) {
                if (rule.covers(ex)) {
                    examplesNotCovered.remove(ex);
                    examplesToCover.remove(ex);
                }
            }
        }

        simplifyRules(classesUnion);
    }

    public final void simplifyRules(SetOfExamples examplesToCover) {
        int simplifiedIndex = 0;
        while (simplifiedIndex < super.size()) {
            Rule tmpRule = get(simplifiedIndex);
            remove(simplifiedIndex);
            if (this.isCompleted(examplesToCover)) {
                --simplifiedIndex;
            } else {
                add(simplifiedIndex, tmpRule);
            }
            ++simplifiedIndex;
        }
    }

    private boolean isCompleted(SetOfExamples examplesToCover) {
        SetOfExamples examplesCovered = new SetOfExamples();
        for (Rule rule : this) {
            for (Example ex : examplesToCover) {
                if (rule.covers(ex)) {
                    examplesCovered.add(ex);
                }
            }
        }
        return examplesToCover.in(examplesCovered);
    }

    public String toString(Latex latex, boolean sortedConditions) {
        StringBuilder sb = new StringBuilder();
        for (Rule rule : this) {
            sb.append(rule.toString(latex, sortedConditions));
            if (latex != Latex.NONE) {
                sb.append(" \\\\\n");
            } else {
                sb.append(" \n");
            }
        }
        return sb.toString();
    }

    @Override
    public String toString() {
        return toString(Latex.NONE, true);
    }
}
